"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _util = _interopRequireDefault(require("../../util"));

var _global = _interopRequireDefault(require("../../global"));

var _scale = _interopRequireDefault(require("./scale"));

var _index = _interopRequireDefault(require("../../attr/index"));

var _colorUtil = _interopRequireDefault(require("../../attr/color-util"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Mapping =
/*#__PURE__*/
function () {
  /** 初始化mapping
    *  初始化mapping
    * @param {*} cfg 配置
    * @param {*} cfg.layer layer对象
    * @param {*} cfg.mesh  mesh对象
    */
  function Mapping(cfg) {
    _classCallCheck(this, Mapping);

    _util["default"].assign(this, cfg);

    if (!this.mesh) this.mesh = this.layer;

    this._init();
  }

  _createClass(Mapping, [{
    key: "_init",
    value: function _init() {
      this._initControllers();

      this._initAttrs();

      this._mapping();
    }
  }, {
    key: "update",
    value: function update() {
      this.mesh.set('scales', {});

      this._initAttrs();

      this._updateMaping();
    }
  }, {
    key: "reMapping",
    value: function reMapping() {
      this.mesh.set('scales', {});

      this._initAttrs();

      this._mapping();
    }
  }, {
    key: "_initControllers",
    value: function _initControllers() {
      var scalesOption = this.layer.get('scaleOptions');
      var scaleController = new _scale["default"]({
        defs: _objectSpread({}, scalesOption)
      });
      this.mesh.set('scaleController', scaleController);
    }
  }, {
    key: "_createScale",
    value: function _createScale(field) {
      var scales = this.mesh.get('scales');

      this._initControllers(); // scale更新


      var scale = scales[field];

      if (!scale) {
        scale = this.createScale(field);
        scales[field] = scale;
      }

      return scale;
    }
  }, {
    key: "createScale",
    value: function createScale(field) {
      var data = this.mesh.layerSource.data.dataArray;
      var scales = this.mesh.get('scales');
      var scale = scales[field];
      var scaleController = this.mesh.get('scaleController');

      if (!scale) {
        scale = scaleController.createScale(field, data);
        scales[field] = scale;
      }

      return scale;
    } // 获取属性映射的值

  }, {
    key: "_getAttrValues",
    value: function _getAttrValues(attr, record) {
      var scales = attr.scales;
      var params = [];

      for (var i = 0; i < scales.length; i++) {
        var scale = scales[i];
        var field = scale.field;

        if (scale.type === 'identity') {
          params.push(scale.value);
        } else {
          params.push(record[field]);
        }
      }

      var indexZoom = params.indexOf('zoom');
      indexZoom !== -1 ? params[indexZoom] = attr.zoom : null;
      var values = attr.mapping.apply(attr, params);
      return values;
    }
  }, {
    key: "_mapping",
    value: function _mapping() {
      var attrs = this.mesh.get('attrs');
      var mappedData = [];
      var data = this.mesh.layerSource.data.dataArray;

      for (var i = 0; i < data.length; i++) {
        var record = data[i];
        var newRecord = {};
        newRecord.id = data[i]._id;

        if (attrs.hasOwnProperty('filter')) {
          var attr = attrs.filter;

          var values = this._getAttrValues(attr, record);

          if (!values[0]) continue;
        }

        for (var k in attrs) {
          if (attrs.hasOwnProperty(k)) {
            var _attr = attrs[k];
            var names = _attr.names;

            var _values = this._getAttrValues(_attr, record);

            if (names.length > 1) {
              // position 之类的生成多个字段的属性
              for (var j = 0; j < _values.length; j++) {
                var val = _values[j];
                var name = names[j];
                newRecord[name] = _util["default"].isArray(val) && val.length === 1 ? val[0] : val; // 只有一个值时返回第一个属性值
              }
            } else {
              newRecord[names[0]] = _values.length === 1 ? _values[0] : _values;
            }
          }
        }

        newRecord.coordinates = record.coordinates;
        mappedData.push(newRecord);
      }

      this.mesh.layerData = mappedData;
    }
  }, {
    key: "_updateMaping",
    value: function _updateMaping() {
      var attrs = this.mesh.get('attrs');
      var data = this.mesh.layerSource.data.dataArray;
      var layerData = this.mesh.layerData;

      for (var i = 0; i < data.length; i++) {
        var record = data[i];

        for (var attrName in attrs) {
          if (attrs.hasOwnProperty(attrName) && attrs[attrName].neadUpdate) {
            var attr = attrs[attrName];
            var names = attr.names;

            var values = this._getAttrValues(attr, record);

            if (names.length > 1) {
              // position 之类的生成多个字段的属性
              for (var j = 0; j < values.length; j++) {
                var val = values[j];
                var name = names[j];
                layerData[i][name] = _util["default"].isArray(val) && val.length === 1 ? val[0] : val; // 只有一个值时返回第一个属性值
              }
            } else {
              layerData[i][names[0]] = values.length === 1 ? values[0] : values;
            }

            attr.neadUpdate = true;
          }
        }
      }
    }
  }, {
    key: "_initAttrs",
    value: function _initAttrs() {
      var attrOptions = this.layer.get('attrOptions');

      for (var type in attrOptions) {
        if (attrOptions.hasOwnProperty(type)) {
          this._updateTileAttr(type);
        }
      }
    }
  }, {
    key: "_updateTileAttr",
    value: function _updateTileAttr(type) {
      var self = this;
      var attrs = this.mesh.get('attrs');
      var attrOptions = this.layer.get('attrOptions');
      var option = attrOptions[type];
      option.neadUpdate = true;

      var className = _util["default"].upperFirst(type);

      var fields = this._parseFields(option.field);

      var scales = [];

      for (var i = 0; i < fields.length; i++) {
        var field = fields[i];

        var scale = self._createScale(field);

        if (type === 'color' && _util["default"].isNil(option.values)) {
          // 设置 color 的默认色值
          option.values = _global["default"].colors;
        }

        scales.push(scale);
      }

      option.scales = scales;
      var attr = new _index["default"][className](option);
      attrs[type] = attr;
    }
  }, {
    key: "_parseFields",
    value: function _parseFields(field) {
      if (_util["default"].isArray(field)) {
        return field;
      }

      if (_util["default"].isString(field)) {
        return field.split('*');
      }

      return [field];
    }
    /**
     * 获取图例配置项
     * @param {*} field 字段
     * @param {*} type 图例类型 color, size
     * @return {*} 图例配置项
     */

  }, {
    key: "getLegendCfg",
    value: function getLegendCfg(field) {
      var _this = this;

      var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'color';

      // todo heatmap
      if (this.layer.type === 'heatmap' && this.layer.shapeType === 'heatmap') {
        return this.get('styleOptions').rampColors;
      }

      var scales = this.layer.get('scales');
      var scale = scales[field];
      var colorAttrs = this.layer.get('attrs')[type];
      var lengendCfg = {};

      if (scale) {
        var ticks = scale.ticks;
        lengendCfg.value = ticks;
        lengendCfg.type = scale.type;
        var values = ticks.map(function (value) {
          var v = _this._getAttrValues(colorAttrs, _defineProperty({}, field, value));

          return type === 'color' ? _colorUtil["default"].colorArray2RGBA(v) : v;
        });
        lengendCfg[type] = values;
      }

      return lengendCfg;
    }
  }]);

  return Mapping;
}();

exports["default"] = Mapping;