"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = TextBuffer;

var _symbolLayout = require("../../../util/symbol-layout");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function TextBuffer(layerData, sourceData, options, fontAtlasManager, collisionIndex, mvpMatrix) {
  var fontWeight = options.fontWeight,
      fontFamily = options.fontFamily;
  var characterSet = [];
  sourceData.forEach(function (element) {
    // shape 存储了 text-field
    var text = element.shape || '';
    text = text.toString();

    for (var j = 0; j < text.length; j++) {
      // 去重
      if (characterSet.indexOf(text[j]) === -1) {
        characterSet.push(text[j]);
      }
    }
  });
  fontAtlasManager.setProps({
    characterSet: characterSet,
    fontFamily: fontFamily,
    fontWeight: fontWeight
  });
  return drawGlyph(layerData, sourceData, options, fontAtlasManager, collisionIndex, mvpMatrix);
}

function drawGlyph(layerData, sourceData, _ref, fontAtlasManager, collisionIndex, mvpMatrix) {
  var _ref$spacing = _ref.spacing,
      spacing = _ref$spacing === void 0 ? 2 : _ref$spacing,
      _ref$textAnchor = _ref.textAnchor,
      textAnchor = _ref$textAnchor === void 0 ? 'center' : _ref$textAnchor,
      _ref$textOffset = _ref.textOffset,
      textOffset = _ref$textOffset === void 0 ? [0, 0] : _ref$textOffset,
      _ref$padding = _ref.padding,
      padding = _ref$padding === void 0 ? [4, 4] : _ref$padding;
  var texture = fontAtlasManager.texture,
      fontAtlas = fontAtlasManager.fontAtlas,
      mapping = fontAtlasManager.mapping;
  var attributes = {
    fontAtlas: fontAtlas,
    texture: texture,
    positions: [],
    colors: [],
    pickingIds: [],
    textUVs: [],
    textOffsets: [],
    textSizes: [],
    index: []
  };
  var indexCounter = 0;
  layerData.forEach(function (feature, i) {
    var size = feature.size,
        coordinates = feature.coordinates; // 根据字段获取文本

    var text = "".concat(layerData[i].shape || ''); // sdf 中默认字号为 24

    var fontScale = size / 24; // 1. 计算每个字符相对锚点的位置

    var shaping = (0, _symbolLayout.shapeText)(text, mapping, 24, textAnchor, 'center', spacing, textOffset);

    if (shaping) {
      // 2. 尝试加入空间索引，获取碰撞检测结果
      // TODO：按照 feature 中指定字段排序，确定插入权重，保证优先级高的文本优先展示
      var _collisionIndex$place = collisionIndex.placeCollisionBox({
        x1: shaping.left * fontScale - padding[0],
        x2: shaping.right * fontScale + padding[0],
        y1: shaping.top * fontScale - padding[1],
        y2: shaping.bottom * fontScale + padding[1],
        // 点要素锚点就是当前点位置
        anchorPointX: coordinates[0],
        anchorPointY: coordinates[1]
      }, mvpMatrix),
          box = _collisionIndex$place.box; // 无碰撞则加入空间索引


      if (box && box.length) {
        // TODO：featureIndex
        collisionIndex.insertCollisionBox(box, 0); // 3. 计算可供渲染的文本块，其中每个字符都包含纹理坐标

        var glyphQuads = (0, _symbolLayout.getGlyphQuads)(shaping, textOffset, false); // 4. 构建顶点数据，四个顶点组成一个 quad

        indexCounter = addAttributeForFeature(feature, attributes, glyphQuads, indexCounter);
      }
    }
  });
  return attributes;
}

function addAttributeForFeature(feature, attributes, glyphQuads, indexCounter) {
  var id = feature.id,
      size = feature.size,
      color = feature.color,
      coordinates = feature.coordinates;
  glyphQuads.forEach(function (quad) {
    var _attributes$colors;

    attributes.pickingIds.push(id, id, id, id);

    (_attributes$colors = attributes.colors).push.apply(_attributes$colors, _toConsumableArray(color).concat(_toConsumableArray(color), _toConsumableArray(color), _toConsumableArray(color)));

    attributes.positions.push(coordinates[0], coordinates[1], coordinates[0], coordinates[1], coordinates[0], coordinates[1], coordinates[0], coordinates[1]);
    attributes.textUVs.push(quad.tex.x, quad.tex.y + quad.tex.height, quad.tex.x + quad.tex.width, quad.tex.y + quad.tex.height, quad.tex.x + quad.tex.width, quad.tex.y, quad.tex.x, quad.tex.y);
    attributes.textOffsets.push(quad.tl.x, quad.tl.y, quad.tr.x, quad.tr.y, quad.br.x, quad.br.y, quad.bl.x, quad.bl.y);
    attributes.textSizes.push(size, size, size, size);
    attributes.index.push(0 + indexCounter, 1 + indexCounter, 2 + indexCounter, 2 + indexCounter, 3 + indexCounter, 0 + indexCounter);
    indexCounter += 4;
  });
  return indexCounter;
}