"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = DrawText;

var THREE = _interopRequireWildcard(require("../../../core/three"));

var _textMaterial = _interopRequireDefault(require("../../../geom/material/textMaterial"));

var _text = _interopRequireDefault(require("../../../geom/buffer/point/text"));

var _colorUtil = _interopRequireDefault(require("../../../attr/color-util"));

var _collisionIndex = _interopRequireDefault(require("../../../util/collision-index"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultTextStyle = {
  fontWeight: 500,
  textAnchor: 'center',
  textOffset: [0, 0],
  spacing: 2,
  padding: [4, 4],
  strokeColor: 'white',
  strokeWidth: 2,
  strokeOpacity: 1.0
};

function DrawText(layerData, layer) {
  var style = _objectSpread({}, defaultTextStyle, {}, layer.get('styleOptions'));

  layer.set('styleOptions', style);
  var activeOption = layer.get('activedOptions');
  var strokeWidth = style.strokeWidth,
      strokeColor = style.strokeColor,
      opacity = style.opacity;

  var _layer$scene$getSize = layer.scene.getSize(),
      width = _layer$scene$getSize.width,
      height = _layer$scene$getSize.height;

  var _updateGeometry2 = _updateGeometry(layerData, layer),
      geometry = _updateGeometry2.geometry,
      texture = _updateGeometry2.texture,
      fontAtlas = _updateGeometry2.fontAtlas;

  layer.scene.on('camerachange', function () {
    var _updateGeometry3 = _updateGeometry(layerData, layer),
        geometry = _updateGeometry3.geometry;

    layer.layerMesh.geometry = geometry;
    layer.layerMesh.geometry.needsUpdate = true;
  });
  var material = new _textMaterial["default"]({
    name: layer.layerId,
    u_sdf_map: texture,
    u_halo_color: _colorUtil["default"].toRGB(strokeColor).map(function (e) {
      return e / 255;
    }),
    u_halo_width: strokeWidth,
    u_halo_blur: 0.5,
    u_font_opacity: opacity,
    u_sdf_map_size: [fontAtlas.width, fontAtlas.height],
    u_viewport_size: [width, height],
    u_activeColor: activeOption.fill
  });
  var mesh = new THREE.Mesh(geometry, material); // 更新 viewport

  window.addEventListener('resize', function () {
    var _layer$scene$getSize2 = layer.scene.getSize(),
        width = _layer$scene$getSize2.width,
        height = _layer$scene$getSize2.height;

    material.uniforms.u_viewport_size.value = [width, height];
    material.uniforms.needsUpdate = true;
  }, false); // 关闭视锥裁剪

  mesh.frustumCulled = false;
  return mesh;
}

function _updateGeometry(layerData, layer) {
  var style = layer.get('styleOptions');
  var fontFamily = style.fontFamily,
      fontWeight = style.fontWeight,
      spacing = style.spacing,
      textAnchor = style.textAnchor,
      textOffset = style.textOffset,
      padding = style.padding;

  var _layer$scene$getSize3 = layer.scene.getSize(),
      width = _layer$scene$getSize3.width,
      height = _layer$scene$getSize3.height;

  var collisionIndex = new _collisionIndex["default"](width, height);
  var _layer$scene$_engine$ = layer.scene._engine._camera,
      projectionMatrix = _layer$scene$_engine$.projectionMatrix,
      matrixWorldInverse = _layer$scene$_engine$.matrixWorldInverse; // 计算 MVP 矩阵

  var mvpMatrix = new THREE.Matrix4().copy(projectionMatrix).multiply(matrixWorldInverse);

  var _ref = new _text["default"](layerData, layerData, {
    fontFamily: fontFamily,
    fontWeight: fontWeight,
    spacing: spacing,
    textAnchor: textAnchor,
    textOffset: textOffset,
    padding: padding
  }, layer.scene.fontAtlasManager, collisionIndex, mvpMatrix),
      index = _ref.index,
      positions = _ref.positions,
      pickingIds = _ref.pickingIds,
      texture = _ref.texture,
      colors = _ref.colors,
      textUVs = _ref.textUVs,
      textOffsets = _ref.textOffsets,
      textSizes = _ref.textSizes,
      fontAtlas = _ref.fontAtlas;

  var geometry = new THREE.BufferGeometry();
  geometry.setIndex(index);
  geometry.addAttribute('a_pos', new THREE.Float32BufferAttribute(positions, 2));
  geometry.addAttribute('pickingId', new THREE.Float32BufferAttribute(pickingIds, 1));
  geometry.addAttribute('a_color', new THREE.Float32BufferAttribute(colors, 4));
  geometry.addAttribute('a_tex', new THREE.Float32BufferAttribute(textUVs, 2));
  geometry.addAttribute('a_offset', new THREE.Float32BufferAttribute(textOffsets, 2));
  geometry.addAttribute('a_size', new THREE.Float32BufferAttribute(textSizes, 1));
  return {
    geometry: geometry,
    texture: texture,
    fontAtlas: fontAtlas
  };
}