"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _gridIndex = _interopRequireDefault(require("../util/grid-index"));

var _three = require("../core/three");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

// 为 viewport 加上 buffer，避免边缘处的文本无法显示
var viewportPadding = 100;
/**
 * 基于网格实现文本避让，大幅提升包围盒碰撞检测效率
 * @see https://zhuanlan.zhihu.com/p/74373214
 */

var CollisionIndex =
/*#__PURE__*/
function () {
  function CollisionIndex(width, height) {
    _classCallCheck(this, CollisionIndex);

    this.width = width;
    this.height = height; // 创建网格索引

    this.grid = new _gridIndex["default"](width + 2 * viewportPadding, height + 2 * viewportPadding, 25);
    this.screenRightBoundary = width + viewportPadding;
    this.screenBottomBoundary = height + viewportPadding;
    this.gridRightBoundary = width + 2 * viewportPadding;
    this.gridBottomBoundary = height + 2 * viewportPadding;
  }

  _createClass(CollisionIndex, [{
    key: "placeCollisionBox",
    value: function placeCollisionBox(collisionBox, mvpMatrix) {
      var projectedPoint = this.project(mvpMatrix, collisionBox.anchorPointX, collisionBox.anchorPointY);
      var tlX = collisionBox.x1 + projectedPoint.x;
      var tlY = collisionBox.y1 + projectedPoint.y;
      var brX = collisionBox.x2 + projectedPoint.x;
      var brY = collisionBox.y2 + projectedPoint.y;

      if (!this.isInsideGrid(tlX, tlY, brX, brY) || this.grid.hitTest(tlX, tlY, brX, brY)) {
        return {
          box: []
        };
      }

      return {
        box: [tlX, tlY, brX, brY]
      };
    }
  }, {
    key: "insertCollisionBox",
    value: function insertCollisionBox(collisionBox, featureIndex) {
      var key = {
        featureIndex: featureIndex
      };
      this.grid.insert(key, collisionBox[0], collisionBox[1], collisionBox[2], collisionBox[3]);
    }
    /**
     * 后续碰撞检测都需要投影到 viewport 坐标系
     * @param {THREE.Matrix4} mvpMatrix mvp矩阵
     * @param {number} x P20 平面坐标X
     * @param {number} y P20 平面坐标Y
     * @return {Point} projectedPoint
     */

  }, {
    key: "project",
    value: function project(mvpMatrix, x, y) {
      var p = new _three.Vector4(x, y, 0, 1).applyMatrix4(mvpMatrix); // GL 坐标系[-1, 1] -> viewport 坐标系[width, height]

      return {
        x: (p.x / p.w + 1) / 2 * this.width + viewportPadding,
        y: (-p.y / p.w + 1) / 2 * this.height + viewportPadding
      };
    }
    /**
     * 判断包围盒是否在整个网格内，需要加上 buffer
     * @param {number} x1 x1
     * @param {number} y1 y1
     * @param {number} x2 x2
     * @param {number} y2 y2
     * @return {Point} isInside
     */

  }, {
    key: "isInsideGrid",
    value: function isInsideGrid(x1, y1, x2, y2) {
      return x2 >= 0 && x1 < this.gridRightBoundary && y2 >= 0 && y1 < this.gridBottomBoundary;
    }
  }]);

  return CollisionIndex;
}();

exports["default"] = CollisionIndex;